@@ -22,6 +22,7 @@
 #include <libgnome/gnome-url.h>
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-init.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
@@ -32,6 +33,9 @@
 #include "gwget_data.h"
 #include "utils.h"
 #include "systray.h"
+#include "md5.h"
+
+#define MD5BUFSIZE 512
 
 static void inform_limit_speed_change(void);
 
@@ -57,7 +61,8 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 	GtkTreeModel *model;
 	GwgetData *gwgetdata;
 	gchar *uri;
-	GError *err = NULL; 
+	GError *err = NULL;
+	GFile *location; 
 	
 	treev=glade_xml_get_widget(xml,"treeview1");
 	select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treev));
@@ -81,10 +86,10 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 				gtk_tree_path_free(path);
 			}
 
-			gwget_data_set_menus(gwget_data_get_selected());
 			popup=glade_xml_get_widget(xml,"popup1");
 			gtk_menu_popup (GTK_MENU(popup), NULL, NULL, NULL, NULL, 
 							event_button->button, event_button->time);
+			gwget_data_set_popupmenu (gwget_data_get_selected());
 			return TRUE;
 		}
 	}
@@ -101,12 +106,15 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 				run_dialog_error(_("Error in download"),gwgetdata->error_msg);
 			} else {
 				if (gwgetdata->recursive) {
-					uri = gnome_vfs_make_uri_from_input(gwgetdata->dir);
+					location = g_file_parse_name (gwgetdata->dir); 
+					uri = g_file_get_uri (location);
+					g_object_unref (location);
 				} else {
-					uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
-                                                 GNOME_VFS_MAKE_URI_DIR_CURRENT);
+					location = g_file_new_for_commandline_arg (gwgetdata->local_filename);
+					uri = g_file_get_uri (location);
+					g_object_unref (location);
 				}
-				if (!gnome_url_show (uri, &err)) {
+				if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, &err)) {
 					run_dialog_error(_("Error opening file"),_("Couldn't open the file"));
 					return FALSE;
 				}
@@ -176,7 +184,6 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 
 	static const gchar *authors[] = { 
 			"David Sedeño Fernández <david@alderia.com>",
-			"Julien Cegarra <julienc@psychologie-fr.org>",
 			NULL
 	};
 		
@@ -187,7 +194,7 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 	gtk_show_about_dialog (NULL,
 			       "name", _("Gwget"), 
 			       "version", VERSION,
-			       "copyright", "Copyright © 2004 David Sedeño Fernández",
+			       "copyright", "Copyright © 2004-2009 David Sedeño Fernández",
 			       "comments", _("Download Manager for GNOME."),
 			       "authors", authors,
 			       "translator-credits", _("translator-credits"),
@@ -214,7 +221,8 @@ new_download(GwgetData* gwgetdata)
 	GtkIconTheme *theme;
 	GtkIconInfo *icon_info;
 	GdkPixbuf *pixbuf;
-	char *mime, *size;
+	gchar *content_type = NULL;
+	gchar *size;
 	int width = 16, height = 16;
 	gdouble perc;
 	
@@ -242,9 +250,9 @@ new_download(GwgetData* gwgetdata)
 	
 	theme = gtk_icon_theme_get_default ();
 	if (!gwgetdata->recursive) {
-		mime = (gchar *)gnome_vfs_get_mime_type_for_name (gwgetdata->local_filename);
+		content_type = g_content_type_guess (gwgetdata->local_filename, NULL, 0, NULL);
 		gwgetdata->icon_name = gnome_icon_lookup (theme, NULL, NULL, NULL, NULL,
-	 							mime, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+	 							content_type, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
 	} else {
 		gwgetdata->icon_name = g_strdup("gtk-refresh");
 	}
@@ -264,7 +272,8 @@ new_download(GwgetData* gwgetdata)
 
 	if (pixbuf)
 		g_object_unref (pixbuf);
-	
+		
+	g_free (content_type);
 }
 
 
@@ -272,7 +281,7 @@ void
 on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 {
 	gchar *xml_file = NULL;
-	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL;
+	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL, *main_window = NULL;
 	
 	if (!xml_pref) {
 		xml_file=g_build_filename(DATADIR,"preferences.glade",NULL);
@@ -280,9 +289,12 @@ on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 		glade_xml_signal_autoconnect(xml_pref);
 	}
 		
+	main_window = glade_xml_get_widget (xml, "main_window");
 	window = glade_xml_get_widget (xml_pref, "pref_window");
 	entry = glade_xml_get_widget(xml_pref,"save_in_entry");
 
+	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(main_window));
+
 	if (gwget_pref.download_dir && strlen(gwget_pref.download_dir)!=0) {
 		gtk_entry_set_text(GTK_ENTRY(entry), gwget_pref.download_dir);
 	} else {
@@ -616,7 +628,7 @@ on_cancel_download_activate(GtkWidget *widget,gpointer data)
 	if (gwgetdata) 
 	{
 		msg = g_strdup_printf(_("Really cancel current download?\n(URL: %s)"),gwgetdata->url);
-		response = run_dialog(_("Delete download?"),_(msg), _("Delete"));
+		response = run_dialog(_("Delete download?"),_(msg), GTK_STOCK_CANCEL, _("Delete"));
 		if (response==GTK_RESPONSE_OK) {
 			gwget_data_stop_download(gwgetdata); 
 			unlink (gwgetdata->local_filename);
@@ -646,7 +658,7 @@ on_remove_completed_activate(GtkWidget *widget, gpointer data)
 	
 	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove completed"),_("Really remove completed downloads from the list?"), _("Remove"));
+		response = run_dialog(_("Remove completed"),_("Really remove completed downloads from the list?"), GTK_STOCK_CANCEL, _("Remove"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -661,6 +673,8 @@ on_remove_completed_activate(GtkWidget *widget, gpointer data)
 					gtk_tree_model_iter_next(model,&iter);
 				}
 			}
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), FALSE);
+			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), _("Gwget - Download Manager"));
 		}
 	}
 				
@@ -678,7 +692,7 @@ on_remove_notrunning_activate(GtkWidget *widget, gpointer data)
 	
 	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove inactive"),_("Really remove inactive downloads from the list?"), _("Remove inactive"));
+		response = run_dialog(_("Remove inactive"),_("Really remove inactive downloads from the list?"), GTK_STOCK_CANCEL, _("Remove inactive"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -710,7 +724,7 @@ on_remove_all_activate(GtkWidget *widget, gpointer data)
 	
 	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove all"),_("Really remove all downloads from the list?"), _("Remove all"));
+		response = run_dialog(_("Remove all"),_("Really remove all downloads from the list?"), GTK_STOCK_CANCEL, _("Remove all"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -740,7 +754,7 @@ on_view_toolbar_activate(GtkWidget *widget,gpointer data)
 	GtkWidget *toolbar,*menu_item;
 	gboolean state;
 	
-	toolbar=glade_xml_get_widget(GLADE_XML(xml),"bonobotoolbar");
+	toolbar=glade_xml_get_widget(GLADE_XML(xml),"toolbar1");
 	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_toolbar");
 	state = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item));
 	
@@ -790,10 +804,118 @@ on_properties_activate(GtkWidget *widget, gpointer data)
 		gtk_label_set_text(GTK_LABEL(local_file),gwgetdata->filename);
 		local_dir=glade_xml_get_widget(GLADE_XML(xml),"local_dir");
 		gtk_label_set_text(GTK_LABEL(local_dir),gwgetdata->dir);
-		gtk_widget_show(properties);
+		if (gwgetdata->state==DL_COMPLETED) {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), TRUE);
+		} else {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), FALSE);
+                }
+                gtk_widget_show(properties);
 	}
 }
 
+void
+on_compare_md5_clicked(GtkWidget *widget, gpointer data)
+{
+   GwgetData *gwgetdata;
+
+   gwgetdata=gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      gtk_widget_show(glade_xml_get_widget(GLADE_XML(xml),"md5_window"));
+   }
+}
+
+void
+on_md5ok_button_clicked(GtkWidget *widget, gpointer data)
+{
+   GtkWidget *md5;
+   GwgetData *gwgetdata;
+   const gchar *entrytext;
+   GtkMessageDialog *dialog;
+   GtkMessageType msg_type;
+
+   gchar *uri;
+   GnomeVFSHandle *fd;
+   GnomeVFSResult result;
+   GString *localfile, *md5res;
+   GString *msg;
+   const gchar *err;
+   gint i;
+   GnomeVFSFileSize fsize;
+   
+   struct md5_ctx md5context;
+   guchar md5digest[16];
+   gchar dataread[MD5BUFSIZE];
+   gboolean close_md5_dlg;
+  
+   gwgetdata = gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      md5=glade_xml_get_widget(GLADE_XML(xml),"md5_window");
+      entrytext=gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(GLADE_XML(xml),"md5_entry")));
+      localfile=g_string_new(gwgetdata->dir);
+      localfile=g_string_append(localfile, gwgetdata->filename); 
+      uri=gnome_vfs_get_uri_from_local_path(localfile->str);
+      result=gnome_vfs_open(&fd, uri, GNOME_VFS_OPEN_READ);
+
+      if (result != GNOME_VFS_OK) {
+         err=gnome_vfs_result_to_string(result);
+         g_printerr(_("error %d when accessing %s: %s\n"), result, uri, err);
+         return;
+      }
+
+      md5_init_ctx(&md5context);
+
+      while(result == GNOME_VFS_OK) {
+         result=gnome_vfs_read(fd, dataread, MD5BUFSIZE, &fsize);  
+         md5_process_bytes(dataread, fsize, &md5context);
+         if (fsize < MD5BUFSIZE)
+            break;
+      }
+
+      md5_finish_ctx(&md5context, md5digest);
+
+      result=gnome_vfs_close(fd);
+      if (result != GNOME_VFS_OK) {
+         err=gnome_vfs_result_to_string(result);
+         g_printerr(_("error %d when accessing %s: %s\n"), result, uri, err);
+         return;
+      }
+
+      md5res=g_string_new("");
+      for (i = 0; i < 16; i++) {
+         g_string_append_printf(md5res,"%02x", md5digest[i]);
+      }
+
+      msg=g_string_new("");
+      if (g_str_equal(md5res->str, entrytext)) {
+         close_md5_dlg=TRUE;
+         g_string_append_printf(msg, _("<span size=\"large\" weight=\"bold\">MD5SUM Check PASSED!</span>"));
+         msg_type=GTK_MESSAGE_INFO;
+      } else {
+         g_string_append_printf(msg, _("<span size=\"large\" weight=\"bold\">MD5SUM Check FAILED!</span>"));
+         close_md5_dlg=FALSE;
+         msg_type=GTK_MESSAGE_WARNING;
+      }
+
+      dialog=(GtkMessageDialog *)gtk_message_dialog_new_with_markup(GTK_WINDOW(md5),
+                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     msg_type,
+                                                     GTK_BUTTONS_CLOSE,
+                                                     "%s", msg->str);
+                                  
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(GTK_WIDGET(dialog));   
+      g_free(uri); 
+      g_string_free(localfile, TRUE);
+      g_string_free(md5res, TRUE);
+      g_string_free(msg, TRUE);
+      if (close_md5_dlg) {
+         gtk_widget_hide(GTK_WIDGET(md5));
+      }
+   }
+}
+
 void 
 on_check_actual_size_toggled(GtkWidget *widget, gpointer data)
 {
@@ -1109,7 +1231,7 @@ on_remove_download_activate (GtkWidget *widget, gpointer data)
 	gwgetdata = gwget_data_get_selected();
 	if (gwgetdata != NULL) {
 	    message = g_strdup_printf (_("Remove %s ?"), gwgetdata->filename);
-		response = run_dialog (message, _("Really remove this download from the list?"), _("Remove download"));
+		response = run_dialog (message, _("Really remove this download from the list?"), GTK_STOCK_CANCEL, _("Remove download"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
